#!/bin/sh
##
##  mkshadow.sh -- create a shadow tree
##
##  Initially written by Ralf S. Engelschall <rse@apache.org>
##  for the shadow tree generation option (--shadow) of 
##  Apache's Autoconf-style Interface (APACI) 
##
#   Heavily modified by Paul Gilmartin <pg@sweng.stortek.com>
#   for OS/390 performance and for correct operation when
#   $src or $dst contains "/../" or involve symbolic links.
#
# This script falls under the Apache License.
# See http://www.apache.org/docs/LICENSE


#   source and destination directory
src="$1"
dst="$2"
: ${SHELL=/bin/sh}

#   check whether source exists
SOURCE=`cd $src && unset PWD && $SHELL -c pwd` || {
    echo "mkshadow.sh:Error: source directory not found" 1>&2
    exit 1
    }

SHADOW=`mkdir -p $dst && cd $dst &&
    unset PWD && $SHELL -c pwd` || exit $?

#   determine if one of the paths is an absolute path,
#   because then we have to use an absolute symlink
oneisabs=0
case $src in
    /* ) oneisabs=1 ;;
esac
case $dst in
    /* ) oneisabs=1 ;;
esac

( cd $src && find . -name CVS -prune   \
                 -o ! -name '*.o'          \
                    ! -name '*.so'         \
                    ! -name '*.so-o'       \
                    ! -name '*.cvsignore'  \
                    ! -name '*.indent.pro' \
                    ! -name '*.apaci*'     \
                    ! -name '*Makefile'    \
                    ! -name '.#*'          \
                    ! -name '*.orig'       \
                    ! -type d              \
                    -print ) |
    "${AWK-awk}" \
      ' BEGIN{ }
	{ if ( ! Back ) {
	    Relative = ( ! oneisabs )
		nSrc = split( SOURCE, Src, "/" )
		nShd = split( SHADOW, Shd, "/" )
		nSame = 0;
		if ( Relative ) for ( I = 1; I <= nSrc; I++ ) {
		    if ( Src[ I ] != Shd [ I ] ) break
		    nSame =  I }

		Back = ""
		if ( Relative )
		    for ( I = nSame + 1; I<=nShd; I++ ) Back = "../"Back
		else Back = ""
	        for ( I = nSame + 1; I<=nSrc; I++ ) Back = Back Src[ I ]"/"
		print( "# Back: " Back "\n" ) }

	  match( $0, "[^/]*$" )
	  Base = substr( $0, RSTART )
	  Dir  = substr( $0, 1, RSTART-2 )"/"
	  sub( "^\\.//*", "", Dir )
	  Levels = split( Dir, Junk, "/" )

	  Dots = Back
	  if ( Relative )
		for ( I = 1; I < Levels; I++ ) Dots = "../"Dots
	  XDir = SHADOW"//"Dir
	  ln[ XDir ] = ln[ XDir ] "    "Dots""Dir""Base" \\\n"
	  rm[ XDir ] = rm[ XDir ] "    "Base" \\\n"
	  ## lv[ XDir ] = " Levels: " Levels " Dots: " Dots " Dir: " Dir
	}
	END{
	    for ( Dir in ln ) {
		print( "echo \"\n: ========= Creating links in:\n: "Dir"\"" )
		printf( "mkdir -p %s//. || exit $?\n", Dir )
		printf( "( cd "Dir" && rm -f \\\n%s%s", rm[ Dir ], ")\n" )
		print( "set -v" )
		printf( "ln -s \\\n%s%s", ln[ Dir ], "  "Dir"//.\n" )
		print( "set +v\n" )
		}
	   }' "SOURCE=$SOURCE" "SHADOW=$SHADOW" "oneisabs=$oneisabs" |
## cat; exit $?
"$SHELL"
